# -*- coding: utf-8 -*-
# AutoSynthesis: The Automated Process Synthesis & Design package.
# Copyright (C) 2022-, Sarang Bhagwat <sarang.bhagwat.git@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# https://github.com/sarangbhagwat/autosynthesis/blob/main/LICENSE
# for license details.
"""
"""

from . import HP_solution_upgrading_acrylic_acid
from .HP_solution_upgrading_acrylic_acid import *
from . import TAL_upgrading_potassium_sorbate
from .TAL_upgrading_potassium_sorbate import *



__all__ = (
    *HP_solution_upgrading_acrylic_acid.__all__,\
    *TAL_upgrading_potassium_sorbate.__all__,
)
